#!/bin/bash
#Description:	install SOS agent

function vercomp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

function checkFileVault () {
    vercomp "${MAJOR_MAC_VERSION}" "10.9"
    if [[ $? -eq 0 ]]; then
        return -1
    fi
    ## Check to see if the encryption process is complete
    encryptCheck=`fdesetup status`
    echo "fdesetup status : ${encryptCheck}"
    statusCheck=$(echo "${encryptCheck}" | grep "FileVault is On.")
    expectedStatus="FileVault is On."
    if [ "${statusCheck}" != "${expectedStatus}" ]; then
        echo "The encryption process has not completed."
        echo "${encryptCheck}"
        return 0
    fi
    
    return 1
}

OPTIND=1
while getopts t:i:m: o
do    case "$o" in
    t)
        REBOOT_OPTION="$OPTARG"
        #0: reboot, 1: switch user
        ;;
    i)
        RES_DIR="$OPTARG"
        #"/Users/Shared/Splashtop SOS Agent/Splashtop SOS.app/Contents/Resources/"
        ;;
    m)
        SETTING_NAME="$OPTARG"
        #sos, sosop
        ;;
    [?])
        echo "invalid option: $1" 1>&2;
        usage "$0"
        exit 1;;
    esac
done
shift $((OPTIND-1)) # Shift off the options and optional --.

[ -z "${SETTING_NAME// }" ] && SETTING_NAME="sos"
[ -z "${RES_DIR// }" ] && RES_DIR="/Users/Shared/Splashtop SOS Agent/Splashtop SOS.app/Contents/Resources"

USER_PLIST="com.splashtop.${SETTING_NAME}-for-user.plist"
ROOT_PLIST="com.splashtop.${SETTING_NAME}-for-root.plist"
AGENT_PLIST="com.splashtop.${SETTING_NAME}-agent.plist"

USER_PATH="/Library/LaunchAgents/"$USER_PLIST
ROOT_PATH="/Library/LaunchAgents/"$ROOT_PLIST
AGENT_PATH="/Library/LaunchDaemons/"$AGENT_PLIST

sudo cp -rf "$RES_DIR/$USER_PLIST" /Library/LaunchAgents
sudo cp -rf "$RES_DIR/$ROOT_PLIST" /Library/LaunchAgents
sudo cp -rf "$RES_DIR/$AGENT_PLIST" /Library/LaunchDaemons

sudo chown root:wheel $ROOT_PATH
sudo chmod 644 $ROOT_PATH

sudo chown root:wheel $USER_PATH
sudo chmod 644 $USER_PATH

sudo chown root:wheel $AGENT_PATH
sudo chmod 644 $AGENT_PATH

ARGUMENTS="-S Aqua"
CURRENTUSER=$(stat -f '%Su' /dev/console)

MAJOR_MAC_VERSION=$(sw_vers -productVersion | awk -F '.' '{print $1 "." $2}')

if [ "$REBOOT_OPTION" == "1" ]; then
    echo "=== Load Streamer agent for "$CURRENTUSER
    #sudo -u $CURRENTUSER killall 'Splashtop SOS Agent'
    
    #sleep 1
    #echo "sudo -u "$CURRENTUSER" launchctl unload "$USER_PATH
    #sudo -u $CURRENTUSER launchctl unload $USER_PATH
    #echo "sudo -u "$CURRENTUSER" killall 'Splashtop SOS'"
    #sudo -u $CURRENTUSER killall 'Splashtop SOS'
    #echo "sudo launchctl unload "$AGENT_PATH
    #sudo launchctl unload $AGENT_PATH
    #echo "sudo launchctl load" $AGENT_PATH
    #sudo launchctl load $AGENT_PATH
    #echo "sudo -u "$CURRENTUSER" launchctl load -w "$ARGUMENTS $USER_PATH
    #sudo -u $CURRENTUSER launchctl load -w $ARGUMENTS $USER_PATH
    #echo "sudo -u "$CURRENTUSER" launchctl start com.splashtop.sos-for-user"
    #sudo -u $CURRENTUSER launchctl start com.splashtop.sos-for-user
else
    echo "=== Reboot Streamer agent for "$CURRENTUSER
    #sudo launchctl unload $AGENT_PATH
    #(sudo launchctl load $AGENT_PATH) &
    
    fileVault="/Users/Shared/SplashtopSOS/filevault.plist"
    checkFileVault
    ret=$?
    echo "Reboot status \"${ret}\""
    if [[ ${ret} -eq 1 ]]; then
        count=1
        while [ $count -lt 5 ]; do
            if [ -f "${fileVault}" ]; then
                echo "FileVault reboot"
                (sleep 1; sudo fdesetup authrestart -inputplist < "${fileVault}") &
                exit 0
                break;
            fi
            count=`expr $count + 1`
            sleep 1
            
            #echo "check \"${fileVault}\" count ${count}"
        done
        echo "Wait 5 sec, FileVault reboot failed!!"
        exit 1
    fi
    
    echo "Normal reboot"
    (sleep 1; sudo shutdown -r now) &
fi

exit 0
